/*	$LAAS$ */
/*
 * Copyright (c) 2005 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */


#include <portLib.h>

#include "server/jlocoHeader.h"

#include "jlocoConst.h"
#include "jlocoPrivate.h"


// Returns 0 on success
static int findTrackedPoster(const char *name, POSTER_ID *posterId);


STATUS
setWatchdogTolerance(int *watchdogTolerance, int *report)
{
  return OK;
}


STATUS
setCmdParam(GENPOS_CART_CMD_PARAM *cmdParam, int *report)
{
  return OK;
}


STATUS
setOdoCorrection(JLOCO_ODO_CORRECTION *tmpOdoCorr, int *report)
{
  if((0 >= tmpOdoCorr->axis_mult)
     || (0 >= tmpOdoCorr->left_mult)
     || (0 >= tmpOdoCorr->right_mult)){
    *report = S_jloco_POSITIVE_VALUE_EXPECTED;
    return ERROR;
  }
  jlocoPrivate->motRr_cor = jlocoPrivate->motRr_nom * tmpOdoCorr->right_mult;
  jlocoPrivate->motRl_cor = jlocoPrivate->motRl_nom * tmpOdoCorr->left_mult;
  jlocoPrivate->motW_cor = jlocoPrivate->motW_nom * tmpOdoCorr->axis_mult;
  return OK;
}


STATUS
setVarParams(JLOCO_VAR_PARAMS *varParams, int *report)
{
  if((0 >= varParams->vxx)
     || (0 >= varParams->vyy)
     || (0 >= varParams->vtt)
     || (0 >= varParams->qCoeff)){
    *report = S_jloco_POSITIVE_VALUE_EXPECTED;
    return ERROR;
  }
  return OK;
}


STATUS
setPos(GENPOS_CART_CONFIG *setPosStr, int *report)
{
  if(JLOCO_TRACK_POSITION == jlocoPrivate->trackMode){
    *report = S_jloco_POS_TRACKING_ON;
    return ERROR;
  }
  initOdometry(jlocoPrivate, setPosStr->x, setPosStr->y, setPosStr->theta,
	       SDI_F->varParams.vxx, SDI_F->varParams.vyy,
	       SDI_F->varParams.vtt, SDI_F->cmdParam.distPoint);
  return OK;
}


STATUS
controlPosErr(GENPOS_CART_CONFIG_VAR *tmpErr, int *report)
{
  memcpy(jlocoPrivate->odo.odoError.var, tmpErr->var,
	 sizeof(jlocoPrivate->odo.odoError.var));
  return OK;
}


STATUS
setGyroMode(int *gyroMode, int *report)
{
  if((JLOCO_GYRO_MODE_ENABLE == *gyroMode)
     || (JLOCO_GYRO_MODE_DISABLE == *gyroMode))
    return OK;
  *report = S_jloco_INVALID_GYRO_MODE;
  return ERROR;
}


STATUS
trackCntrl(JLOCO_NAME *posterName, int *report)
{
  if(0 != findTrackedPoster(posterName->str, &jlocoPrivate->trackedPoster)){
    *report = S_jloco_POSTER_NOT_FOUND;
    return ERROR;
  }
  // NOTE: jlocoPrivate->trackMode = JLOCO_TRACK_POS is done in the
  // jlocoPumpTaskCodels.c:trackExec(), after it has verified the
  // external watchdog.
  return OK;
}


STATUS
trackSpeedCntrl(JLOCO_NAME *posterName, int *report)
{
  if(0 != findTrackedPoster(posterName->str, &jlocoPrivate->trackedPoster)){
    *report = S_jloco_POSTER_NOT_FOUND;
    return ERROR;
  }
  // NOTE: jlocoPrivate->trackMode = JLOCO_TRACK_SPEED is done in the
  // jlocoPumpTaskCodels.c:trackSpeedExec(), after it has verified the
  // external watchdog.
  return OK;
}


int findTrackedPoster(const char *name, POSTER_ID *posterId)
{
  POSTER_ID id;
  if(OK != posterFind((char*) name, &id)){ // posterFind() should use const
    PDEBUG("posterFind(%s) failed\n", name);
    return -1;
  }
  *posterId = id;
  return 0;
}
