/*
 * Little bits of calife's libmat copied here to avoid linking with it
 *
 * XXXXX should be replaced some day...
 */

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include "libmat.h"

#define MATELT(mat,i,j,c)   ( *((mat) + (c) * (i) + (j)) )
#define INDEX_SMATELT(i,j)   \
((i)<(j) ? ((j)*((j)+1))/2 + (i) : ((i)*((i)+1))/2 +(j)) 
#define SMATELT(mat,i,j,n) *((mat) + INDEX_SMATELT((i),(j)))
#define MATSIZE(l,c)        ((l) * (c))
#define SMATSIZE(n,c)       (((n) * ((n) + 1)) / 2)

/***********************************************************************

  dms_fill (mat, l, c, coeff1, coeff2, ..., coeffn)
  
  remplissage d'une matrice symetrique l * c  (l = c !)
  
  - Entree : - mat = pointeur sur doubles (matrice)
             - l, c = nbre de lignes, nbre de colonnes (doivent etre egaux !)
             - coeffi = les coeffs de la matrice trianguaire superieure, 
	                ligne apres ligne.


  Ex : dms_fill (mat, 3, 3, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0)
 
             [ 1.0 2.0 3.0 ]
  -->  mat = [ 2.0 4.0 5.0 ]
             [ 3.0 5.0 6.0 ]

  En fait, le stockage interne est :
  mat = [ 1.0 2.0 4.0 3.0 5.0 6.0 ] (pour compatibilite avec IMSL)
----------------------------------------------------------------------*/

int 
dms_fill(double *mat, int l, int c, ...)
{
  va_list ap;
  int i, j;

  va_start(ap, c);

  if (l != c) {
	    fprintf(stderr, "dms : matrice symetrique non carree !!!\n");
	    return (1);
  }

  for (i = 0; i < l; i++)
    for (j = i; j < c; j++)
      *(mat + (j*(j+1))/2 + i) = (double) va_arg (ap, double);

  va_end (ap);
  return (0);
}


int 
dms_plus(const double *m1, const double *m2, double *m_res, int l, int c)
{
	int k;

	if (l != c)
	{
		fprintf (stderr, "dms : matrice symetrique non carree !!!\n");
		return -1;
	}

	for (k = 0; k < SMATSIZE(l, c); k++)
		*m_res++ = *m1++ + *m2++;
	return 0;
}

int
dmfsf_jmult2 (const double *m1, const double *m2, double *m_res,
		int l1, int c1, int l2, int c2)
{

	double *temp, *res;
	double *aux = NULL;
	double *aux2 = NULL;
	int i,j,k,flag = 0;
	double val;

	if (l2 != c2 || c1 != c2)
	{
		fprintf(stderr, "%s : tailles incorrectes \n", __func__);
		return -1;
	}

	if (m_res == m1 || m_res == m2)
	{
		temp = malloc((l1*c2 + l1*(l1+1)/2) * sizeof(double));
		if (temp == NULL) {
			fprintf(stderr, "%s : Can't allocate memory for temp\n", __func__);
			return -1;
		}
		aux = temp;
		res = temp + l1*c2;
		aux2 = res;
		flag = 1;
	}
	else
	{
		temp = malloc((l1 * c2) * sizeof(double));
		if (temp == NULL) {
			fprintf(stderr, "%s : Can't allocate memory for temp\n", __func__);
			return -1;
		}
		aux = temp;
		res = m_res;
	}


	for (i = 0; i < l1; i++)
		for (j = 0; j < c2; j++)
		{
			val = 0.;
			for (k = 0; k < c1; k++)
				val += MATELT (m1, i, k, c1) * SMATELT (m2, k, j, c2);
			*temp++ = val;
		}
	temp = aux;

	for (j = 0; j < l1; j++)
		for (i = 0; i <= j; i++)
		{
			val = 0.;
			for (k = 0; k < c2; k++)
				val += MATELT (temp, i, k, c2) * MATELT (m1, j, k, c1);
			*res++ = val;
		}

	if (flag)
	{
		res = aux2;
		for (i = 0; i < SMATSIZE (l1, l1); i++)
			*m_res++ = *res++;
	}

	free(temp);

	return 0;
}

