/*	$LAAS$ */
/*
 * Copyright (c) 2005 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#ifndef _ODOPROBA_H
#define _ODOPROBA_H


#include <genPosStruct.h>


typedef struct JLOCO_ROBOT_MISC {
  double s;                       // curvilenear abscissus
  double cR;                      // curvature
  double initTheta;               // initial theta
  double dTheta;                  // theta variation
  double dS;                      // s variation
  double sinTheta;                // sinus of theta
  double cosTheta;                // guess what ? cosinus of theta !
} JLOCO_ROBOT_MISC;


typedef struct JLOCO_WHEEL_PARAM {
  double phi;                     // angular position
  double w;                       // angular speed
  double v;                       // translation speed
  double dPhi;                    // angular variation
  double dS;                      // linear variation
  int nRot;                       // number of rotations
  int unused;                     // padding
} JLOCO_WHEEL_PARAM;


typedef struct JLOCO_ODO {
  GENPOS_CART_STATE robot;          // position
  GENPOS_CART_CONFIG_VAR odoError;  // uncertainty
  JLOCO_ROBOT_MISC robotMisc;       // cached values (eg sin(theta) etc)
  JLOCO_WHEEL_PARAM rightWheel;     // movement state of wheel
  JLOCO_WHEEL_PARAM leftWheel;
} JLOCO_ODO;


typedef struct JLOCO_GYRO {
  GENPOS_CART_STATE robot;         // position
  GENPOS_CART_CONFIG_VAR odoError; // uncertainty
  JLOCO_ROBOT_MISC robotMisc;      // cached values (eg sin(theta) etc)
} JLOCO_GYRO;


//////////////////////////////////////////////////
// Update probabilistic odometry
//
// IN:  posId - old pose and covariance
//              wheel displacement measurements
//      lax   - wheelbase
//      coeff - uncertainty coefficient of wheel rotation
//
// OUT: posId - new position and covariance
//
void odoProba0(JLOCO_ODO *posId, double lax, double coeff);


//////////////////////////////////////////////////
// Update odometry in mixed wheel / gyro mode
//
// IN:  odoId -  old covariance and some wheel displacement data:
//               odoId->robotMisc.dS
//               odoId->robot.v
//               odoId->odoError.var
//      gyroId - old position and angle measure from gyroscope:
//               gyroId->robotMisc.dTheta
//               gyroId->robot.theta
//               
// OUT: gyroId - new position and covariance:
//               gyroId->robot.{theta,sinTheta,cosTheta}
//               gyroId->robotMisc.{dS,s}
//               gyroId->robot.v
//               gyroId->robot.{xC,yC}
//               gyroId->robotMisc.cR
//               gyroId->odoError.var
//
// TODO: Implement error propagation!
//
void odoGyroProba(JLOCO_ODO *odoId, JLOCO_GYRO *gyroId,  double lax);


#endif // _ODOPROBA_H
