/*	$LAAS$ */
/*
 * Copyright (c) 2005 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */


#ifndef _JLOCOSTRUCT_H
#define _JLOCOSTRUCT_H


#include "jlocoConst.h"


typedef struct JLOCO_NAME {
  char str[JLOCO_STRLEN];
} JLOCO_NAME;


// Default values for resetting the robot position and uncertainty of
// the measured wheel increments.
typedef struct JLOCO_VAR_PARAMS {
  double vxx;			// initial variance along x [m2]
  double vyy;			// initial variance along y [m2]
  double vtt;			// initial variance for theta [rad2]
  double qCoeff;		// uncertainty factor of wheel increment
} JLOCO_VAR_PARAMS;


// odometry model correction factors, applied to wheel base and wheel
// radii individually.
typedef struct JLOCO_ODO_CORRECTION {
  double axis_mult;		// factor on wheel base
  double left_mult;		// factor on left wheel radius
  double right_mult;		// factor on right wheel radius
} JLOCO_ODO_CORRECTION;


typedef enum JLOCO_MODE {
  JLOCO_IDLE = 0,		// smoothly go to or keep standstill
  JLOCO_MANUAL,			// smoothly follow joystick commands
  JLOCO_TRACK_SPEED,		// track speed poster
  JLOCO_TRACK_POSITION,		// track position poser
  JLOCO_EMERGENCY		// force immediate standstill
} JLOCO_MODE;

#endif
